/**
 * OWASP AppSensor
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * AppSensor project. For details, please see
 * <a href="http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project">
 * 	http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project</a>.
 *
 * Copyright (c) 2010 - The OWASP Foundation
 * 
 * AppSensor is published by OWASP under the BSD license. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author Michael Coates <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton <a href="http://www.jtmelton.com/">jtmelton</a>
 * @created 2010
 */
package org.owasp.appsensor;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.owasp.esapi.EncryptedProperties;
import org.owasp.esapi.SecurityConfiguration;
import org.owasp.esapi.errors.EncryptionException;
import org.owasp.esapi.reference.DefaultSecurityConfiguration;
import org.owasp.esapi.reference.crypto.DefaultEncryptedProperties;

/**
 * This class simply extends the DefaultSecurityConfiguration from ESAPI
 * in order to ensure we can recognize a few extra configuration options 
 * that are specific to AppSensor.  These include the configuration for 
 * the response action and intrusion store implementations as well as 
 * the configuration of the duration and timescale fields for disabling
 * components.  Other than that, everything from the DefaultSecurityConfiguration
 * still exists.   
 * 
 * 9/14/2010 - jtm - had to add singleton config since esapi 2.0rc7 changed their 
 * management code for instantiation of singletons. This is related to bug: 
 * http://code.google.com/p/appsensor/issues/detail?id=3
 * 
 * @author Michael Coates (michael.coates .at. owasp.org) 
 *         <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton (jtmelton .at. gmail.com)
 *         <a href="http://www.jtmelton.com/">jtmelton</a>
 * @since February 24, 2010
 * @see org.owasp.esapi.reference.DefaultSecurityConfiguration
 */
public class AppSensorSecurityConfiguration extends DefaultSecurityConfiguration {
    private static volatile SecurityConfiguration singletonInstance;

    public static SecurityConfiguration getInstance()
    {
        if ( singletonInstance == null ) {
            synchronized ( AppSensorSecurityConfiguration.class ) {
                if ( singletonInstance == null ) {
                    singletonInstance = new AppSensorSecurityConfiguration();
                }
            }
        }
        return singletonInstance;
    }
	
	/** properties for querying */
	private static Properties combinedProperties = null;
	
	/** properties file lookup key for response action implementation */
	private static final String RESPONSE_ACTION_IMPLEMENTATION = "AppSensor.responseAction";
	/** properties file lookup key for intrusion store implementation */
	private static final String INTRUSION_STORE_IMPLEMENTATION = "AppSensor.intrusionStore";
	/** properties file lookup key for AppSensor Utilities implementation */
	private static final String ASUTILITIES_IMPLEMENTATION = "AppSensor.asUtilities";
	/** properties file lookup key for AppSensor Utilities implementation */
	private static final String TREND_LOGGER_IMPLEMENTATION = "AppSensor.trendLogger";
	
	/** default implementation class for response action implementation */
	private static final String DEFAULT_RESPONSE_ACTION_IMPLEMENTATION = "org.owasp.appsensor.intrusiondetection.DefaultResponseAction";
	/** default implementation class for intrusion store implementation */
	private static final String DEFAULT_INTRUSION_STORE_IMPLEMENTATION = "org.owasp.appsensor.intrusiondetection.DefaultIntrusionStore";
	/** default implementation class for AppSensor Utilities implementation */
	private static final String DEFAULT_ASUTILITIES_IMPLEMENTATION = "org.owasp.appsensor.reference.DefaultASUtilities";
	/** default implementation class for AppSensor Utilities implementation */
	private static final String DEFAULT_TREND_LOGGER_IMPLEMENTATION = "org.owasp.appsensor.trendmonitoring.reference.InMemoryTrendLogger";
	
	/** Name of encrypted properties lookup string that should be in appsensor.properties file */
	public static final String APPSENSOR_ENCRYPTED_PROPERTIES_FILE_LOOKUP_KEY = "appsensor.encrypted.properties.file";
	
	/**
	 * Constructor that loads the configuration on class creation
	 */
	public AppSensorSecurityConfiguration() {
		if (combinedProperties == null) {
			loadAppSensorConfiguration();
		}
	}

	/**
	 * This method is not implemented in this class.  getAppSensorQuota() should be used instead.
	 * @param eventName event name
	 * @return Threshold object
	 */
	@Override
	public Threshold getQuota(String eventName) {
        throw new UnsupportedOperationException("This method is not implemented in this class.");
    }

	/**
	 * This method retrieves the configuration information from the ESAPI.properties file
	 * for managing when and how to respond to an intrusion or set of intrusions.
	 * @param eventName event code to lookup
	 * @return AppSensorThreshold object containing intrusion detection configuration information
	 */
	public AppSensorThreshold getAppSensorQuota(String eventName) {
        int count = getProperty("IntrusionDetector." + eventName + ".count", 0);
        int interval =  getProperty("IntrusionDetector." + eventName + ".interval", 0);
        List<String> defaultActions = new ArrayList<String>();
        List<String> actions = getProperty("IntrusionDetector." + eventName + ".actions", defaultActions);
        
        int disableComponentDuration = getProperty("IntrusionDetector." + eventName + ".disableComponent.duration", 0);
        String disableComponentTimeScale = getProperty("IntrusionDetector." + eventName + ".disableComponent.timeScale", "s");
        
        int disableComponentForUserDuration = getProperty("IntrusionDetector." + eventName + ".disableComponentForUser.duration", 0);
        String disableComponentForUserTimeScale = getProperty("IntrusionDetector." + eventName + ".disableComponentForUser.timeScale", "s");        
        
		if (! AppSensorServiceMonitor.acceptableTimeScales.contains(disableComponentTimeScale)) {
			//if we got here, it means someone has requested an unsupported timeScale
			throw new IllegalArgumentException("There has been a request for an timeScale " +
					"that is not supported by this service monitor.  The requested timeScale is: " + disableComponentTimeScale);
		}
        
        if ( count > 0 && interval > 0 && actions.size() > 0 ) {
        	return new AppSensorThreshold(eventName, count, interval, actions, 
        			disableComponentDuration, disableComponentTimeScale, 
        			disableComponentForUserDuration, disableComponentForUserTimeScale);  
        }
        return null;
    }
	
	/**
	 * This method returns the exceptions list that should not be considered 
	 * when determining which components to block ... ie. these components
	 * should always be active.  The default implementation uses request URIs 
	 * to represent the components.
	 * @return list of components that are exceptions to being blocked. (should always be available)
	 */
	public List<String> getDisableComponentExceptionsList() {
		List<String> emptyList = new ArrayList<String>();
		List<String> exceptions = getProperty("disable.component.exceptions", emptyList);
		return exceptions;
	}
	
	/**
	 * This method returns the collection of stored strings that appear to be obvious XSS attacks 
	 * @return collection of stored strings that appear to be obvious XSS attacks 
	 */
	public List<String> getXSSAttackPatternsList() {
		List<String> emptyList = new ArrayList<String>();
		List<String> attacks = getProperty("xss.attack.patterns", emptyList);
		return attacks;
	}
	
	/**
	 * This method returns the collection of stored strings that appear to be obvious SQL Injection attacks 
	 * @return collection of stored strings that appear to be obvious SQL Injection attacks 
	 */
	public List<String> getSQLInjectionAttackPatternsList() {
		List<String> emptyList = new ArrayList<String>();
		List<String> attacks = getProperty("sql.injection.attack.patterns", emptyList);
		return attacks;
	}
	
	/**
	 * This method returns the collection of stored strings that represent all possible HTTP methods per the HTTP 1.1 spec
	 * @return collection of stored strings that represent all possible HTTP methods per the HTTP 1.1 spec
	 */
	public List<String> getAllHttpMethods() {
		List<String> emptyList = new ArrayList<String>();
		List<String> methods = getProperty("all.http.methods", emptyList);
		return methods;
	}
	
	/**
	 * This method returns the collection of stored strings that are considered valid somewhere in this application
	 * @return collection of stored strings that are considered valid somewhere in this application
	 */
	public List<String> getValidHttpMethods() {
		List<String> emptyList = new ArrayList<String>();
		List<String> methods = getProperty("valid.http.methods", emptyList);
		return methods;
	}
	
	/**
	 * This method returns the extensions list that should be considered 
	 * when determining which components to block ... ie. these extensions
	 * should be checked to determine if they should be blocked.  The default 
	 * implementation uses request URIs to represent the components. 
	 * So, for instance, your list might contain .jsp, .do, .action, etc.
	 * This would not be a place typically to include static resources like
	 * images, css, javascript, etc.
	 * @return list of components that are exceptions to being blocked. (should always be available)
	 */
	public List<String> getDisableComponentExtensionsToCheckList() {
		List<String> emptyList = new ArrayList<String>();
		List<String> exceptions = getProperty("disable.component.extensionsToCheck", emptyList);
		return exceptions;
	}
	
	/**
	 * This method gets the class name of the class configured as the 
	 * appsensor response action for this application.   
	 * @return the class name of the class configured as the 
	 * 		appsensor response action for this application.   
	 */
	public String getResponseActionImplementation() {
		return getProperty(RESPONSE_ACTION_IMPLEMENTATION, DEFAULT_RESPONSE_ACTION_IMPLEMENTATION);
	}
	
	/**
	 * This method gets the class name of the class configured as the 
	 * appsensor intrusion store for this application.   
	 * @return the class name of the class configured as the 
	 * 		appsensor intrusion store for this application.  
	 */
	public String getIntrusionStoreImplementation() {
		return getProperty(INTRUSION_STORE_IMPLEMENTATION, DEFAULT_INTRUSION_STORE_IMPLEMENTATION);
	}
	
	/**
	 * This method gets the class name of the class configured as the 
	 * appsensor utilities for this application.   
	 * @return the class name of the class configured as the 
	 * 		appsensor utilities for this application.  
	 */
	public String getASUtilitiesImplementation() {
		return getProperty(ASUTILITIES_IMPLEMENTATION, DEFAULT_ASUTILITIES_IMPLEMENTATION);
	}
	
	/**
	 * This method gets the class name of the class configured as the 
	 * appsensor trend logger for this application.   
	 * @return the class name of the class configured as the 
	 * 		appsensor trend logger for this application.  
	 */
	public String getTrendLoggerImplementation() {
		return getProperty(TREND_LOGGER_IMPLEMENTATION, DEFAULT_TREND_LOGGER_IMPLEMENTATION);
	}
	
	/**
	 * Retrieves the email protocol to use when sending to SMS clients
	 * @return the email protocol to use when sending to SMS clients
	 */
	public String getSMSEmailProtocol() {
		return getProperty("smsAdmin.protocol", "smtp");
	}
	
	/**
	 * Retrieves the email host to use when sending to SMS clients
	 * @return the email host to use when sending to SMS clients
	 */
	public String getSMSEmailHost() {
		return getProperty("smsAdmin.host", "");
	}
	
	/**
	 * Retrieves the email port to use when sending to SMS clients
	 * @return the email port to use when sending to SMS clients
	 */
	public int getSMSEmailPort() {
		return getProperty("smsAdmin.port", 25);
	}
	
	/**
	 * Retrieves whether or not the email host requires authentication when sending to SMS clients
	 * @return whether or not the email host requires authentication when sending to SMS clients
	 */
	public boolean getSMSEmailAuthenticationRequired() {
		return getProperty("smsAdmin.mail.host.authentication.required", false);
	}
	
	/**
	 * Retrieves the authenticating user to use when sending to SMS clients
	 * @return the authenticating user to use when sending to SMS clients
	 */
	public String getSMSEmailAuthenticationUser() {
		return getProperty("smsAdmin.send.user", "");
	}
	
	/**
	 * Retrieves the authenticating password to use when sending to SMS clients
	 * @return the authenticating password to use when sending to SMS clients
	 */
	public String getSMSEmailAuthenticationPassword() {
		return getProperty("smsAdmin.send.password", "");
	}
	
	/**
	 * Retrieves the email from account to use when sending to SMS clients
	 * @return the email from account to use when sending to SMS clients
	 */
	public String getSMSEmailFromAccount() {
		return getProperty("smsAdmin.from", "");
	}
	
	/**
	 * Retrieves the email to account(s) to use when sending to SMS clients
	 * @return the email to account(s) to use when sending to SMS clients
	 */
	public List<String> getSMSEmailToAccounts() {
		List<String> emptyList = new ArrayList<String>();
		return getProperty("smsAdmin.to", emptyList);
	}
	
	/**
	 * Retrieves the email protocol to use when sending to email clients
	 * @return the email protocol to use when sending to email clients
	 */
	public String getEmailProtocol() {
		return getProperty("emailAdmin.protocol", "smtp");
	}
	
	/**
	 * Retrieves the email host to use when sending to email clients
	 * @return the email host to use when sending to email clients
	 */
	public String getEmailHost() {
		return getProperty("emailAdmin.host", "");
	}
	
	/**
	 * Retrieves the email port to use when sending to email clients
	 * @return the email port to use when sending to email clients
	 */
	public int getEmailPort() {
		return getProperty("emailAdmin.port", 25);
	}
	
	/**
	 * Retrieves whether or not the email host requires authentication when sending to email clients
	 * @return whether or not the email host requires authentication when sending to email clients
	 */
	public boolean getEmailAuthenticationRequired() {
		return getProperty("emailAdmin.mail.host.authentication.required", false);
	}
	
	/**
	 * Retrieves the authenticating user to use when sending to email clients
	 * @return the authenticating user to use when sending to email clients
	 */
	public String getEmailAuthenticationUser() {
		return getProperty("emailAdmin.send.user", "");
	}
	
	/**
	 * Retrieves the authenticating password to use when sending to email clients
	 * @return the authenticating password to use when sending to email clients
	 */
	public String getEmailAuthenticationPassword() {
		return getProperty("emailAdmin.send.password", "");
	}
	
	/**
	 * Retrieves the email from account to use when sending to email clients
	 * @return the email from account to use when sending to email clients
	 */
	public String getEmailFromAccount() {
		return getProperty("emailAdmin.from", "");
	}
	
	/**
	 * Retrieves the email to account(s) to use when sending to email clients
	 * @return the email to account(s) to use when sending to email clients
	 */
	public List<String> getEmailToAccounts() {
		List<String> emptyList = new ArrayList<String>();
		return getProperty("emailAdmin.to", emptyList);
	}
	
	/////////////////////////
	
	/**
	 * This method performs in initial loading of the properties files including
	 * ESAPI.properties and appsensor.properties and makes these properties available
	 * to the class. 
	 * @throws IOException exception that may be thrown if there is an error loading properties files 
	 */
	private void loadAppSensorConfiguration() {
		try {
			combinedProperties = loadPropertiesFromStream( getResourceStream( "ESAPI.properties" ) );
		} catch(IOException ioe) {
			try {
				combinedProperties = loadConfigurationFromClasspath("ESAPI.properties");
			} catch (Exception e) {		
				//ignored
			}	
		}
    	
		Properties appsensorProperties = null;
    	
    	try {
    		appsensorProperties = loadPropertiesFromStream( getResourceStream( "appsensor.properties" ) );
		} catch(IOException ioe) {
			try {
				appsensorProperties = loadConfigurationFromClasspath("appsensor.properties");
			} catch (Exception e) {		
				//ignored
			}	
		}

		if (appsensorProperties != null) {
			for (Object o : appsensorProperties.keySet()) {
	    		String key = (String)o;
	    		String value = appsensorProperties.getProperty(key);
	    		combinedProperties.put( key, value);
	    	}
		}
		
		String encryptedPropertiesFile = combinedProperties.getProperty(APPSENSOR_ENCRYPTED_PROPERTIES_FILE_LOOKUP_KEY);
		
		//if encrypted properties file is configured in appsensor.properties - try to load that file and it's properties
		if (encryptedPropertiesFile != null && encryptedPropertiesFile.trim().length() > 0) {
			EncryptedProperties appsensorEncryptedProperties = null;
	    	
	    	try {
	    		appsensorEncryptedProperties = loadEncryptedPropertiesFromStream( getResourceStream( encryptedPropertiesFile ) );
			} catch(IOException ioe) {
				try {
					appsensorEncryptedProperties = loadEncryptedConfigurationFromClasspath(encryptedPropertiesFile);
				} catch (Exception e) {		
					//ignored
				}	
			}
	
			if (appsensorEncryptedProperties != null) {
				for (Object o : appsensorEncryptedProperties.keySet()) {
					try {
			    		String key = (String)o;
			    		String value = appsensorEncryptedProperties.getProperty(key);
			    		combinedProperties.put( key, value);
					} catch (EncryptionException e) {
						throw new IllegalArgumentException("Failed to load key '" + o 
								+ "' as a from encrypted properties file: " + encryptedPropertiesFile);
					}
		    	}
			}
		}
	}
	
	/** 
	 * This method is copied from ESAPIs DefaultSecurityConfiguration class.
	 * It simply loads the properties file from the specified input stream. 
	 * @param is input stream for properties file
	 * @return Properties object representing all properties from file
	 * @throws IOException exception that may be thrown if there is an error loading properties files 
	 */
    private Properties loadPropertiesFromStream( InputStream is) throws IOException {
    	Properties config = new Properties();
        try {
	        config.load(is);
        } finally {
            if ( is != null ) try { is.close(); } catch( Exception e ) {}
        }
        return config;
    }
	
	/** 
	 * This method is like loadPropertiesFromStream, but simply loads the properties 
	 * from an encrypted file from the specified input stream. 
	 * @param is input stream for properties file
	 * @return Properties object representing all properties from file
	 * @throws IOException exception that may be thrown if there is an error loading properties files 
	 */
    private EncryptedProperties loadEncryptedPropertiesFromStream( InputStream is) throws IOException {
    	EncryptedProperties config = new DefaultEncryptedProperties();
        try {
	        config.load(is);
        } finally {
            if ( is != null ) try { is.close(); } catch( Exception e ) {}
        }
        return config;
    }
    
    /**
     * Used to load properties files from a variety of different classpath locations.
     * 
     * @param fileName The properties file filename.
     * @return loaded properties
     * @throws IllegalArgumentException exception if resource isn't loaded
     */
	private Properties loadConfigurationFromClasspath(String fileName) throws IllegalArgumentException {
		Properties result = null;
		InputStream in = null;

		ClassLoader[] loaders = new ClassLoader[] {
				Thread.currentThread().getContextClassLoader(),
				ClassLoader.getSystemClassLoader(),
				getClass().getClassLoader() 
		};

		ClassLoader currentLoader = null;
		for (int i = 0; i < loaders.length; i++) {
			if (loaders[i] != null) {
				currentLoader = loaders[i];
				try {
					// try root
					in = loaders[i].getResourceAsStream(fileName);
					
					// try .esapi folder
					if (in == null) {
						in = currentLoader.getResourceAsStream(".esapi/" + fileName);
					} 
					
					// try resources folder
					if (in == null) {
						in = currentLoader.getResourceAsStream("resources/" + fileName);
					}
		
					// now load the properties
					if (in != null) {
						result = new Properties();
						result.load(in); // Can throw IOException
					}
				} catch (Exception e) {
					result = null;
		
				} finally {
					try {
						in.close();
					} catch (Exception e) {
					}
				}
			}
		}

		if (result == null) {
		    throw new IllegalArgumentException("Failed to load " + fileName + " as a classloader resource.");
		}

		return result;
	}

    /**
     * Used to load encrypted properties files from a variety of different classpath locations.
     * 
     * @param fileName The encrypted properties file filename.
     * @return loaded decrypted properties
     * @throws IllegalArgumentException exception if resource isn't loaded
     */
	private EncryptedProperties loadEncryptedConfigurationFromClasspath(String fileName) throws IllegalArgumentException {
		EncryptedProperties result = null;
		InputStream in = null;

		ClassLoader[] loaders = new ClassLoader[] {
				Thread.currentThread().getContextClassLoader(),
				ClassLoader.getSystemClassLoader(),
				getClass().getClassLoader() 
		};

		ClassLoader currentLoader = null;
		for (int i = 0; i < loaders.length; i++) {
			if (loaders[i] != null) {
				currentLoader = loaders[i];
				try {
					// try root
					in = loaders[i].getResourceAsStream(fileName);
					
					// try .esapi folder
					if (in == null) {
						in = currentLoader.getResourceAsStream(".esapi/" + fileName);
					} 
					
					// try resources folder
					if (in == null) {
						in = currentLoader.getResourceAsStream("resources/" + fileName);
					}
		
					// now load the properties
					if (in != null) {
						result = new DefaultEncryptedProperties();
						result.load(in); // Can throw IOException
					}
				} catch (Exception e) {
					result = null;
		
				} finally {
					try {
						in.close();
					} catch (Exception e) {
					}
				}
			}
		}

		if (result == null) {
		    throw new IllegalArgumentException("Failed to load " + fileName + " as a classloader resource.");
		}

		return result;
	}
    
    /**
     * This method is copied from ESAPIs DefaultSecurityConfiguration class.
     * It simply pulls the specified property from the properties object and
     * returns it as a string.  The defaultValue is used if the key does not 
     * locate a value in the properties file. 
     * @param key lookup value
     * @param defaultValue default value to use if the key is not found 
     * @return String representing property value
     */
	public String getProperty( String key, String defaultValue ) {
		String value = combinedProperties.getProperty(key);
		if ( value == null ) {
    		return defaultValue;
		}
		return value.trim();
	}
	
	/**
     * This method is copied from ESAPIs DefaultSecurityConfiguration class.
     * It simply pulls the specified property from the properties object and
     * returns it as a boolean.  The defaultValue is used if the key does not 
     * locate a value in the properties file. 
     * @param key lookup value
     * @param defaultValue default value to use if the key is not found 
     * @return boolean representing property value
     */
	public boolean getProperty( String key, boolean defaultValue ) {
		String value = combinedProperties.getProperty(key);
		if ( value == null ) {
    		return defaultValue;
		}
		try {
            return Boolean.parseBoolean( value );
		} catch( NumberFormatException e ) {
			return defaultValue;
		}
	}
	
	/**
	 * This method is copied from ESAPIs DefaultSecurityConfiguration class.
	 * It simply pulls the specified property from the properties object and
     * returns it as an integer.  The defaultValue is used if the key does not 
     * locate a value in the properties file. 
	 * @param key lookup value
	 * @param defaultValue default value to use if the key is not found 
	 * @return integer representing property value
	 */
	private int getProperty( String key, int defaultValue ) {
		String property = combinedProperties.getProperty(key);
		if ( property == null ) {
    		return defaultValue;
		}
		try {
            return Integer.parseInt( property );
		} catch( NumberFormatException e ) {
			return defaultValue;
		}
	}
	
	/**
	 * Returns a List representing the parsed, comma-separated property
	 * @param key lookup value
	 * @param defaultValue default value to use if the key is not found 
	 * @return List of strings representing property values
	 */
	private List<String> getProperty( String key, List<String> defaultValue ) {
		String property = combinedProperties.getProperty( key );
		if ( property == null ) {
    		return defaultValue;
		}
		String[] parts = property.split(",");
		List<String> partsList = new ArrayList<String>();
		for (String part : parts) {
			partsList.add(part.trim());
		}
        return partsList;
	}

}
