/**
 * OWASP AppSensor
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * AppSensor project. For details, please see
 * <a href="http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project">
 * 	http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project</a>.
 *
 * Copyright (c) 2010 - The OWASP Foundation
 * 
 * AppSensor is published by OWASP under the BSD license. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author Michael Coates <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton <a href="http://www.jtmelton.com/">jtmelton</a>
 * @created 2010
 */
package org.owasp.appsensor.intrusiondetection;

import java.util.List;

import org.owasp.appsensor.APPSENSOR;
import org.owasp.appsensor.ASLogger;
import org.owasp.appsensor.AppSensorIntrusion;
import org.owasp.appsensor.AppSensorSecurityConfiguration;
import org.owasp.appsensor.AppSensorThreshold;
import org.owasp.appsensor.errors.AppSensorException;
import org.owasp.appsensor.errors.AppSensorSystemException;
import org.owasp.esapi.IntrusionDetector;
import org.owasp.esapi.errors.EnterpriseSecurityException;
import org.owasp.esapi.errors.IntrusionException;

/**
 * Implementation of the IntrusionDetector interface. This
 * implementation monitors both EnterpriseSecurityExceptions as well as 
 * AppSensorExceptions to see if any user exceeds a configurable threshold in 
 * a configurable time period. For example, it can monitor to see if a user exceeds 
 * 10 input validation issues in a 1 minute period. Or if there are more than 3 
 * authentication problems in a 10 second period. 
 * <p>
 * This implementation stores state in the configured IntrusionStore class.  
 * The default implementation of the IntrusionStore (DefaultIntrusionStore) 
 * maintains the information "in-memory" so that information does not persist 
 * across application restarts.  However, more complex implementations that store 
 * the information in a file, database or other data store are certainly possible.
 * <p>
 * In order to use this class, it must be configured in the ESAPI.properties file.
 * <p>
 * It is possible to override the IntrusionStore that holds the intrusion related information, 
 * or to override the ResponseAction which responds to events at the appropriate point. 
 * In order to do so, you need to modify the configuration in the appsensor.properties file.
 * <p>
 * Finally, this class uses the AppSensorSecurityConfiguration and AppSensorThreshold 
 * classes to ensure it has the appropriate information from the configuration files in order
 * to make decisions.
 * 
 * 9/14/2010 - jtm - had to add singleton config since esapi 2.0rc7 changed their 
 * management code for instantiation of singletons. This is related to bug: 
 * http://code.google.com/p/appsensor/issues/detail?id=3
 * 
 * @author Michael Coates (michael.coates .at. owasp.org) 
 *         <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton (jtmelton .at. gmail.com)
 *         <a href="http://www.jtmelton.com/">jtmelton</a>
 * @since February 24, 2010
 * @see org.owasp.esapi.IntrusionDetector
 */
public class AppSensorIntrusionDetector implements org.owasp.esapi.IntrusionDetector {

    private static volatile IntrusionDetector singletonInstance;

    public static IntrusionDetector getInstance()
    {
        if ( singletonInstance == null ) {
            synchronized ( AppSensorIntrusionDetector.class ) {
                if ( singletonInstance == null ) {
                    singletonInstance = new AppSensorIntrusionDetector();
                }
            }
        }
        return singletonInstance;
    }
    
	//DefaultIntrusionDetector implementation from ESAPI can be found here 
	//http://www.google.com/codesearch/p?hl=en&sa=N&cd=4&ct=rc#N8fUvf0LCCY/trunk/src/main/java/org/owasp/esapi/reference/DefaultIntrusionDetector.java&q=threshold%20package:http://owasp-esapi-java\.googlecode\.com&l=33
	
	/** Logger */
	private final static ASLogger logger = APPSENSOR.asUtilities().getLogger("AppSensorIntrusionDetector");
	
	/** Intrusion storage */
	private static IntrusionStore intrusionStore = null;
	
	/** Response handler */
	private static ResponseAction responseAction = null;
	 
	/** Implementation of security config that pulls appropriate fields from appsensor.properties */
	private static AppSensorSecurityConfiguration assc = new AppSensorSecurityConfiguration();
	
	/**
	 * Constructor that initializes the intrusion store and response action 
	 */
	public AppSensorIntrusionDetector() {
		if (responseAction == null) {
			responseAction = APPSENSOR.responseAction();
		}
		
		if (intrusionStore == null) {
			intrusionStore = APPSENSOR.intrusionStore();
		}
	}
	
	/**
	 * 
	 * Constructor that initializes the intrusion store and response action by 
	 * taking request parameters
	 *
	 * @param responseAction specified response action implementation
	 * @param intrusionStore specified intrusion store implementation
	 */
	public AppSensorIntrusionDetector(ResponseAction responseAction, IntrusionStore intrusionStore) {
		AppSensorIntrusionDetector.responseAction = responseAction;
		AppSensorIntrusionDetector.intrusionStore = intrusionStore;
	}

	/**
	 * {@inheritDoc}
	 */
	public void addEvent(String eventName, String logMessage) throws IntrusionException {
        logger.warning("Security event " + eventName + " received with message: " + logMessage);
        logger.warning("Forwarding on Security event " + eventName + " as an AppSensorException");
        
        //instead of writing separate handler code, we're just going to go ahead and treat 
        //an event as an AppSensorException since creating one will simply cause the addExceptionMethod below
        //to be called, though with a bit of redirection
        new AppSensorException(eventName, logMessage, logMessage);
	}

	/**
	 * {@inheritDoc}
	 */
	public void addException(Exception e) throws IntrusionException {
		//don't handle intrusion exceptions, it could get into a loop
		if ( e instanceof IntrusionException) {
            return;
        }
		
		//handle logging for specific exception types if possible, else default to just using the message
		if ( e instanceof EnterpriseSecurityException ) {
            logger.warning(((EnterpriseSecurityException)e).getLogMessage(), e );
        } else if (e instanceof AppSensorException) {
        	logger.warning(((AppSensorException)e).getLogMessage(), e );
        } else if (e instanceof AppSensorSystemException) {
        	logger.warning(((AppSensorSystemException)e).getLogMessage(), e );
        } else {
            logger.warning(e.getMessage(), e );
        }
		
		//add exception to storage, and get the created AppSensorIntrusion
		AppSensorIntrusion asIntrusion = intrusionStore.addExceptionToIntrusionStore(e);
		
		//Here we need to check if the exception we just added was a system exception
		//we should grab the intrusion record for the system user, otherwise we'll 
		//grab the intrusion record for the current user
		IntrusionRecord intrusionRecord;
		if (asIntrusion.getSecurityException() instanceof AppSensorSystemException) {
			//get the system user IntrusionRecord containing all of the system user's intrusions
			intrusionRecord = intrusionStore.getIntrusionRecordForSystemUser();
		} else {
			//get the user specific IntrusionRecord containing all of this user's intrusions
			intrusionRecord = intrusionStore.getIntrusionRecordForCurrentUser();
		}

		// check if user has violated thresholds
		// 1. Check if the user has passed the limit for this type of event
		// 2. Then check if the user has passed the limit for total security events
		if (checkUserViolation(intrusionRecord, asIntrusion.getEventCode())) {
			//went over threshold on specific event
			takeSecurityAction(intrusionRecord, asIntrusion);
		} else if (checkUserViolation(intrusionRecord, IntrusionRecord.ALL_INTRUSIONS)) {
			//went over threshold on all events
			takeSecurityAction(intrusionRecord, asIntrusion);
		}

	}

	/**
	 * This method checks the current number of per user intrusions to see if a threshold 
	 * has been crossed for either all intrusions or this specific intrusion.  If so,
	 * the event is responded to.
	 * @param intrusionRecord per user object holding intrusion information
	 * @param eventCode code for event / detection point
	 * @return if the threshold has been crossed
	 */
	private boolean checkUserViolation(IntrusionRecord intrusionRecord, String eventCode) {
		AppSensorThreshold threshold;
		Long interval;
		try {
			threshold = getAppSensorThreshold(eventCode);
			
			interval = threshold.interval;
		} catch (NullPointerException e) {
			// no violation configured for this specific event, still will count towards total violations
			return false; 
		}

		//get number of intrusions for this eventCode
		int numIntrusions = 0;
		if (IntrusionRecord.ALL_INTRUSIONS.equalsIgnoreCase(eventCode)) {
			numIntrusions = intrusionRecord.getNumberOfAllIntrusions();
		} else {
			numIntrusions = intrusionRecord.getNumberOfIntrusions(eventCode);
		}
		
		String logEntry = "";
		logEntry = "Checking for user violation: EventCode: " + eventCode + ", ";
		logEntry = logEntry + "Threshold: " + threshold.name + ", Allowed Limit: " + threshold.count + ", ";
		logEntry = logEntry + "Total Events Recorded: " + numIntrusions;

		logger.info(logEntry);

		//get the number of these events that have occurred in the last interval (ie. 30 seconds)
		int initialCount = intrusionRecord.getNumberOfIntrusionsInInterval(eventCode, interval);
		int count = initialCount;
		
		//4 examples for the below code
		//1. count is 5, t.count is 10 (5%10 = 5, reset count to 5 which is < 10 prints No Violation and returns false)
		//2. count is 45, t.count is 10 (45%10 = 5, reset count to 5 which is < 10 prints No Violation and returns false) 
		//3. count is 10, t.count is 10 (10%10 = 0, no reset 10 >= 10 prints Violation Observed, adds violation and returns true)
		//4. count is 30, t.count is 10 (30%10 = 0, no reset 30 >= 10 prints Violation Observed, adds violation and returns true)

		if (count % threshold.count != 0) {
			count = count % threshold.count;// mod count with limit - this prevents firing on event 11 when count is every 10
		}
		
		logger.debug("Total Overall Events Qualifying " + initialCount);
		logger.debug("Total Events Qualifying For Threshold " + count);
		
		if (count >= threshold.count) {
			logger.info("Violation Observed for id <" + intrusionRecord.getUserID() + ">");
			intrusionRecord.addViolation(eventCode);
			return true;
		} else {
			logger.info("No Violation for id <" + intrusionRecord.getUserID() + ">");
			return false;
		}

	}
	
	/**
 	 * This method decides which response action to take based on those available and those 
 	 * already executed.  It then dispatches the ResponseAction to perform the actual response.
	 * @param intrusionRecord per user object holding intrusion information
	 * @param currentIntrusion object representing current intrusion.
	 */
	private void takeSecurityAction(IntrusionRecord intrusionRecord, AppSensorIntrusion currentIntrusion) {
		AppSensorThreshold threshold = null;
		String lastViolation = null;
		
		//locking to get last violation .. this will always be populated since the  
		//checkUserViolation method is called as a guard to this method and it is the 
		//method that actually sets the last violation
		synchronized (this) {
			if (intrusionRecord.getLastViolation() == null) {
				throw new IllegalArgumentException("The last Violation is null - this SHOULD NOT happen!  " +
						"It should be set as part of determining the threshold has been crossed which occurs " +
						"before taking security action");
			}
			lastViolation = intrusionRecord.getLastViolation();
		}
		
		try {
			// get threshold for this event
			threshold = getAppSensorThreshold(lastViolation);
		} catch (Exception e) {
			logger.warning("No threshhold defined for" + lastViolation);
			// no threshold configured for this event, default to "Total" exception threshold
			threshold = getAppSensorThreshold(IntrusionRecord.ALL_INTRUSIONS);
		}
		
		//get the response actions configured for this type of event 
		List<String> responseActions = threshold.actions;
		
		logger.debug("Possible response actions for this event are: " + responseActions);
		
		//get the last action that has been performed if any
		//ie. if "log, logout, disable" is configured, maybe "log" has already been performed
		String lastAction = intrusionRecord.getLastResponseAction(currentIntrusion.getEventCode());
		
		int nextActionIndex = 0;
		if (lastAction != null) {
			//pull the location of the last response, and go up one to grab the next in line
			nextActionIndex = responseActions.indexOf(lastAction) + 1;
			logger.debug("Last executed response was: " + lastAction);
		}
		
		//if the last action was the most recently executed, perform it again
		if (nextActionIndex >= responseActions.size()) {
			nextActionIndex = responseActions.size() - 1;
			logger.error("No further response actions defined. Repeating last action of '" + responseActions.get(nextActionIndex) + "'");
		}
		
		String action = responseActions.get(nextActionIndex);
		intrusionRecord.setLastResponseAction(action, currentIntrusion.getEventCode());

		responseAction.handleResponse(action, currentIntrusion);
	}

	/**
	 * Retrieves the AppSensorThreshold from the security config class for the detection point 
	 * with the given eventCode 
	 * @param eventCode code for event / detection point
	 * @return AppSensorThreshold holding threshold information from the ESAPI.properties file
	 * 		   for the given eventCode
	 */
	public static AppSensorThreshold getAppSensorThreshold(String eventCode) {
		AppSensorThreshold threshold = assc.getAppSensorQuota(eventCode);
		return threshold;
	}
	
	/**
	 * Simple getter
	 * @return intrusion store
	 */
	public IntrusionStore getIntrusionStore() {
		return intrusionStore;
	}
	
	/**
	 * Simple getter
	 * @return response action
	 */
	public ResponseAction getResponseAction() {
		return responseAction;
	}
}
