/**
 * OWASP AppSensor
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * AppSensor project. For details, please see
 * <a href="http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project">
 * 	http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project</a>.
 *
 * Copyright (c) 2010 - The OWASP Foundation
 * 
 * AppSensor is published by OWASP under the BSD license. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author Michael Coates <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton <a href="http://www.jtmelton.com/">jtmelton</a>
 * @created 2010
 */
package org.owasp.appsensor.intrusiondetection.reference;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import org.owasp.appsensor.AppSensorIntrusion;
import org.owasp.appsensor.DateUtils;
import org.owasp.appsensor.intrusiondetection.IntrusionRecord;

/**
 * This class represents the collection of intrusions for a specific user.
 * These records are managed by and housed in the intrusion store.  
 * <p>
 * The essential ways of interacting with this class once it is created
 * are to add intrusions and violations as events/attacks occur and to 
 * query this class for information about intrusions and violations 
 * that have occurred. 
 * 
 * @author Michael Coates (michael.coates .at. owasp.org) 
 *         <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton (jtmelton .at. gmail.com)
 *         <a href="http://www.jtmelton.com/">jtmelton</a>
 * @since September 16, 2010
 */
public class DefaultIntrusionRecord implements IntrusionRecord {

	/** User this record is for */
	private String userID;
	
	/** Container that holds all IntrusionExceptions thrown by this user account */
	private Collection<AppSensorIntrusion> intrusionsCommitted = new ArrayList<AppSensorIntrusion>();
	
	/** Container that holds the number of violations, ie crossing of given thresholds */
	private Collection<String> violations = new ArrayList<String>();
	
	/** Most recent violation */
	private String lastViolation = null;
	
	/** Most recent response */
	private String DEFAULT_LAST_RESPONSE_ACTION = "NONE";
	
	//jtm - 5/23/2011 - change to concurrent map to address http://code.google.com/p/appsensor/issues/detail?id=6
	private ConcurrentHashMap<String, LinkedList<String>> lastResponseActionMap = new ConcurrentHashMap<String, LinkedList<String>>();

	/**
	 * Constructor to create record for specified user
	 * @param userID userId to create intrusion record for
	 */
	public DefaultIntrusionRecord(String userID) {
		this.userID = userID;
	}

	/**
	 * {@inheritDoc}
	 */
	public void addIntrusionToRecord(AppSensorIntrusion aie) {
		intrusionsCommitted.add(aie);
	}

	/**
	 * {@inheritDoc}
	 */
	public int getNumberOfAllIntrusions() {
		return getNumberOfIntrusions(ALL_INTRUSIONS);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public int getNumberOfIntrusions(String eventCode) {
		int defaultValue = 0;
		int count = 0;

		//if the code is for total - just get number for all intrusions
		if ((ALL_INTRUSIONS).equalsIgnoreCase(eventCode)) {
			count = intrusionsCommitted.size();
		} else {
			//handle app sensor specific intrusions individually
			//if app-sensor exception, check event code and return all of that type
			for (AppSensorIntrusion asi : intrusionsCommitted) {
				if (asi != null && asi.getEventCode() != null && eventCode != null) {
					//if we get a match increment count
					if (eventCode.equalsIgnoreCase(asi.getEventCode())) {
						count++;
					}
				}
				
			}//end loop
		}
		if (count < defaultValue) {
			// Avoid the rare integer overflow scenario
			count = Integer.MAX_VALUE;
		}
		return count;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getUserID() {
		return this.userID;
	}

	/**
	 * {@inheritDoc}
	 */
	public void addViolation(String eventCode) {
		long currentTime = DateUtils.getCurrentTime();

		String entry = String.valueOf(currentTime) + "," + eventCode;
		violations.add(entry);
		lastViolation = eventCode;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getLastViolation() {
		return lastViolation;
	}

	/**
	 * {@inheritDoc}
	 */
	public void setLastResponseAction(String lastResponseAction, String eventCode) {
		//create default list to use in case there's not one in the map
		LinkedList<String> lastResponses = new LinkedList<String>();
		
		if (lastResponseActionMap.containsKey(eventCode)) {
			//search for the specified event code
			for (String ec : lastResponseActionMap.keySet()) {
				if (ec != null && ec.equals(eventCode)) {
					//use map list if available - already has contents
					lastResponses = lastResponseActionMap.get(eventCode);
					//update map with most recent response action
					lastResponses.add(lastResponseAction);
				}
			}
		} else {
			//there's no existing record, starting from scratch
			//add response
			lastResponses.add(lastResponseAction);
			//add to map
			lastResponseActionMap.putIfAbsent(eventCode, lastResponses);
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String getLastResponseAction(String eventCode) {
		//use default in case last response doesn't exist yet
		String lastResponse = DEFAULT_LAST_RESPONSE_ACTION;
		
		//checking if code exists in map
		if (lastResponseActionMap.containsKey(eventCode)) {
			//search for the specified event code
			for (String ec : lastResponseActionMap.keySet()) {
				if (ec != null && ec.equals(eventCode)) {
					LinkedList<String> lastResponses = lastResponseActionMap.get(eventCode);
					//return last response
					lastResponse = lastResponses.getLast();
				}
			}
		} 
		return lastResponse;
	}

	/**
	 * {@inheritDoc}
	 */
	public int getNumberOfIntrusionsInInterval(String eventCode, Long intervalInSeconds) {
		//set the begin time for considering events to the current time minus the specified time interval in seconds
		long beginTime = DateUtils.getTimeNumberOfSecondsAgo(intervalInSeconds);
		
		int count = 0;

		//loop through intrusions incrementing count as we find matches
		for (AppSensorIntrusion asi : intrusionsCommitted) {
			Long airEventTime = asi.getTimeStamp();
			if (airEventTime > beginTime) {
				if (eventCode.equalsIgnoreCase(ALL_INTRUSIONS) || asi.getEventCode().equalsIgnoreCase(eventCode)) {
					//increment if we're considering either all intrusions or if we find the specific one we're looking for
					count++;
				}
			}
		}
		return count;
	}

	/**
	 * {@inheritDoc}
	 */
	public List<AppSensorIntrusion> getCopyIntrusionsObserved() {
		List<AppSensorIntrusion> asiCopy = new ArrayList<AppSensorIntrusion>();
		for (AppSensorIntrusion asi : intrusionsCommitted) {
			asiCopy.add(asi);
		}
		return asiCopy;
	}
} 
