/**
 * OWASP AppSensor
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * AppSensor project. For details, please see
 * <a href="http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project">
 * 	http://www.owasp.org/index.php/Category:OWASP_AppSensor_Project</a>.
 *
 * Copyright (c) 2010 - The OWASP Foundation
 * 
 * AppSensor is published by OWASP under the BSD license. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author Michael Coates <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton <a href="http://www.jtmelton.com/">jtmelton</a>
 * @created 2010
 */
package org.owasp.appsensor.trendmonitoring.reference;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.owasp.appsensor.trendmonitoring.TrendEvent;

/**
 * This class is simply a data store for trend events.  There are a few accessor 
 * methods that allow access to the data in a few key ways. 
 * <p>
 * This data store is simply a proof of concept data store that allows the trend monitoring concepts
 * to be displayed.  While this data store will technically work in your application, 
 * no effort has been put into optimizing it for use.  It will continuously grow in size
 * as the application is used until restarted, meaning performance is likely poor.
 * 
 * @author Michael Coates (michael.coates .at. owasp.org) 
 *         <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @author John Melton (jtmelton .at. gmail.com)
 *         <a href="http://www.jtmelton.com/">jtmelton</a>
 * @since June 17, 2010
 */
public class InMemoryTrendDataStore {
	private static List<TrendEvent> events = null;

	public synchronized void addEvent(TrendEvent event) {
		initializeEventsIfNecessary();
		events.add(event);
	}

	public synchronized List<TrendEvent> getCopyOfAllEvents() {
		initializeEventsIfNecessary();
		List<TrendEvent> tempEvents = new ArrayList<TrendEvent>();
		tempEvents.addAll(events);
		return tempEvents;
	}
	
	public synchronized HashMap<String, List<TrendEvent>> getCopyOfAllEventsByResource() {
		initializeEventsIfNecessary();
		HashMap<String, List<TrendEvent>> eventsByResource = new HashMap<String, List<TrendEvent>>();
		
		//loop through all events
		for (TrendEvent te : events) {
			String resource = te.getResourceAccessed();
			//if resource is already in map
			if (eventsByResource.containsKey(resource)) {
				List<TrendEvent> tempEvents = eventsByResource.get(resource);
				tempEvents.add(te);
				eventsByResource.put(resource, tempEvents);
			} else {	//resource is not yet in map
				List<TrendEvent> tempEvents = new ArrayList<TrendEvent>();
				tempEvents.add(te);
				eventsByResource.put(resource, tempEvents);
			}
		}
		
		return eventsByResource;
	}
	
	public synchronized HashMap<String, List<TrendEvent>> getCopyOfAllEventsByUserName() {
		initializeEventsIfNecessary();
		HashMap<String, List<TrendEvent>> eventsByUserName = new HashMap<String, List<TrendEvent>>();
		
		//loop through all events
		for (TrendEvent te : events) {
			String userName = te.getAccessedByName();
			//if resource is already in map
			if (eventsByUserName.containsKey(userName)) {
				List<TrendEvent> tempEvents = eventsByUserName.get(userName);
				tempEvents.add(te);
				eventsByUserName.put(userName, tempEvents);
			} else {	//resource is not yet in map
				List<TrendEvent> tempEvents = new ArrayList<TrendEvent>();
				tempEvents.add(te);
				eventsByUserName.put(userName, tempEvents);
			}
		}
		
		return eventsByUserName;
	}
	
	public synchronized HashMap<String, List<TrendEvent>> getCopyOfAllEventsByUserAddress() {
		initializeEventsIfNecessary();
		HashMap<String, List<TrendEvent>> eventsByUserAddress = new HashMap<String, List<TrendEvent>>();
		
		//loop through all events
		for (TrendEvent te : events) {
			String userAddress = te.getAccessedByAddress();
			//if resource is already in map
			if (eventsByUserAddress.containsKey(userAddress)) {
				List<TrendEvent> tempEvents = eventsByUserAddress.get(userAddress);
				tempEvents.add(te);
				eventsByUserAddress.put(userAddress, tempEvents);
			} else {	//resource is not yet in map
				List<TrendEvent> tempEvents = new ArrayList<TrendEvent>();
				tempEvents.add(te);
				eventsByUserAddress.put(userAddress, tempEvents);
			}
		}
		
		return eventsByUserAddress;
	}
	
	private void initializeEventsIfNecessary() {
		if (events == null) {	//only happens once
			events = new ArrayList<TrendEvent>();
		}
	}
	
	// singleton code below
	public static InMemoryTrendDataStore getInstance() {
		return InMemoryTrendDataStoreHolder.instance;
	}

	private InMemoryTrendDataStore() {
	}

	private static final class InMemoryTrendDataStoreHolder {
		static final InMemoryTrendDataStore instance = new InMemoryTrendDataStore();
	}
}
